#!/usr/bin/env perl
#
# NAME:
#	getdata.pl - extract SOA and A records from DNS dump
#
# SYNOPSIS:
#	getdata.pl "named_dump"
#
# DESCRIPTION:
#	This script reads "named_dump" and outputs SOA records for
#	each domain followed by PTR,NS,MX and A records.
#	The output looks like:
#
#.nf
#	"type key data"
#
#	for example:
#
#	SOA crufty.net ns.crufty.net hostmaster@crufty.net
#	SOA 250.12.203.in-addr.arpa ns.crufty.net
#	PTR 1.250.12.203.in-addr.arpa gate.crufty.net
#	PTR 130.250.12.203.in-addr.arpa gate.crufty.net
#	NS crufty.net ns.crufty.net
#	MX crufty.net gate.crufty.net 100
#	A ns.crufty.net 203.12.250.1
#	A gate.crufty.net 203.12.250.1
#	A gate.crufty.net 203.12.250.130
#
#.fi
#
#	Which makes further automated processing such as generating
#	reverse maps, quite simple.
#
# NOTES:
#	To simplify down stream processing "type" is always in UPPER
#	case and all other output is in lower case.  We attempt to
#	ignore bogus data especially in PTR records.
#
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>
#

# RCSid:
#	$Id: getdata.pl,v 1.13 2008/01/26 04:25:46 sjg Exp $
#
#	@(#)Copyright (c) 1996-2001 Simon J. Gerraty.
#	All rights reserved.
#	
#	This software is the property of Simon J. Gerraty.
#	It may be used only in accordance with the conditions outlined
#	in the accompanying LICENSE file. If your distribution does
#	not contain a LICENSE file, then you have no rights to use
#	this software and should contact sales@crufty.net to obtain a
#	licensed distribution.
#	
#	This software is provided "AS IS" in the hope that it will be
#	of use.  Since it is not possible to anticipate all possible
#	use of this software, there is absolutely NO WARRANTY.
#
#	Please forward bug reports and fixes to help@crufty.net
#

if ($0 =~ m,^(.*)/([^/]+)$,) {
  $Mydir = $1;
  $Myname = $2;
} else {
  $Mydir = '.';
  $Myname = $0;
}
require "$Mydir/getopts.pl";

&Getopts('X:');

sub main {
  open(STDOUT, "| sort -r") || die;
  
  while (<>) {
    s/;.*//;
    next unless (m/\w/);
    y/A-Z/a-z/;
    s/(\S)\.\s/$1 /g;
    s/(\S)\.$/$1/;
    
    if (m/\$origin\s+(\S+)/) {
      $origin = ".$1";
      if ($origin eq '..') {
	$origin = '';
      }
      next;
    }
    if (m/^(\S+)/) {
      $host = $1;
    }
    # BIND9 is the future, and its different
    if (m/\s\d+\s+soa\s+(\S+)\s+(\S+)/) {
      $ns=$1;
      $email=$2;
      $email =~ s/\\*\@/./;	# make sure it is legal
      $email =~ s/\./\@/;	# now usable
      print "SOA $host$origin $ns $email\n";
    } elsif (m/\s\d+\s+ns\s+(\S+)/) {
      print "NS $host$origin $1\n";
    } elsif (m/\s\d+\s+aaaa\s+(\S+)/) {
      print "AAAA $host$origin $1\n";
    } elsif (m/\s\d+\s+a\s+(\S+)/) {
      print "A $host$origin $1\n";
    } elsif (m/^([1-9][\d.]*\S+)\s.*\d+\s+ptr\s+(\S+)/) {
      print "PTR $1$origin $2\n";
    } elsif (m/\s\d+\s+mx\s+(\d+)\s+(\S+)/) {
      print "MX $host$origin $2 $1\n";
    } elsif ($opt_X =~ m/C/o && m/\s\d+\s+cname\s+(\S+)/) {
      print "CNAME $host$origin $1\n";
    } elsif ($opt_X =~ m/T/o && m/\s\d+\s+txt\s+(\S+.*)/) {
      print "TXT $host$origin $1\n";

    # BIND8 and earlier
    } elsif (m/\sin\s+soa\s+(\S+)\s+(\S+)/) {
      $ns=$1;
      $email=$2;
      $email =~ s/\\*\@/./;	# make sure it is legal
      $email =~ s/\./\@/;	# now usable
      print "SOA $host$origin $ns $email\n";
    } elsif (m/\sin\s+ns\s+(\S+)/) {
      print "NS $host$origin $1\n";
    } elsif (m/\sin\s+a\s+(\S+)/) {
      print "A $host$origin $1\n";
    } elsif (m/^([1-9][\d.]*)\s.*in\s+ptr\s+(\S+)/) {
      print "PTR $1$origin $2\n";
    } elsif (m/\sin\s+mx\s+(\d+)\s+(\S+)/) {
      print "MX $host$origin $2 $1\n";
    } elsif ($opt_X =~ m/C/o && m/\sin\s+cname\s+(\S+)/) {
      print "CNAME $host$origin $1\n";
    } elsif ($opt_X =~ m/T/o && m/\sin\s+txt\s+(\S+.*)/) {
      print "TXT $host$origin $1\n";
    }
  }

  close STDOUT;
  exit 0;
}

&main;


